package ch.zhaw.ads;

import java.util.Random;
import java.util.function.Consumer;

public class SortServer implements CommandExecutor {

    @Override
    public String execute(String command) {

        String[] command_split = command.split(" ");
        int testdataSize = Integer.parseInt(command_split[1]);

        int[] testdata = new int[testdataSize];
        Random random = new Random();
        for (int i=0; i < testdata.length; i++) {
            testdata[i] = random.nextInt(1000000);
        }

        double runtime = 0;
        if (command_split[0].toLowerCase().startsWith("bubble")) {
            runtime = timeit(this::BubbleSort, testdata);
            BubbleSort(testdata);
        } else if (command_split[0].toLowerCase().startsWith("insert")) {
            runtime = timeit(this::InsertSort, testdata);
            InsertSort(testdata);
        } else if (command_split[0].toLowerCase().startsWith("select")) {
            runtime = timeit(this::SelectionSort, testdata);
            SelectionSort(testdata);
        }

        return "Is sorted: " + checkSorted(testdata) + ", runtime " + runtime + "ms\n";
    }

    private void BubbleSort(int[] a) {
        for (int k = a.length-1; k > 0; k--) {
            boolean noSwap = true;
            for (int i = 0; i < k; i++){
                if ( a[i] > a[i+1]) {
                    int temp = a[i];
                    a[i] = a[i+1];
                    a[i+1] = temp;
                    noSwap = false;
                }
            }
            if (noSwap) break;
        }
    }

    private void SelectionSort(int[] a) {
        for (int k = 0; k < a.length; k++){
            int min = k;
            for (int i = k+1; i < a.length; i ++) {
                if (a[i] < a[min]) min = i;
            }
            if (min != k) {
                int temp = a[min];
                a[min] = a[k];
                a[k] = temp;
            }
        }
    }

    private void InsertSort(int[] a) {
        for (int k = 1; k < a.length; k++)
            if (a[k] < a[k-1]){
                int x = a[k];
                int i;
                for (i = k; ((i > 0) && (a[i-1] > x));i--)
                    a[i] = a[i-1];
                a[i] = x;
            }
    }

    private boolean checkSorted(int[] a) {
        for (int i=1; i < a.length; i++) {
            if (a[i-1] > a[i]) return false;
        }
        return true;
    }

    private double timeit(Consumer<int[]>func, int[] a) {
        long end, start = System.currentTimeMillis();
        int count = 0;
        int[] testarray = new int[a.length];
        do {
            System.arraycopy(a, 0, testarray, 0, a.length);
            func.accept(testarray);
            count++;
            end = System.currentTimeMillis();
        } while (end - start < 1000);
        return (double)(end-start)/count;
    }
}
